/****************************************************************************
 * tx.cs.native_mbwc
 * Implemntation of tx.charset.NativeCharset using C's multibyte/widechar
 * functions. <<<Assuming that wchar_t is either in UTF16 or in UTF32.>>>
 * If it is not the case, consider using tx.cs.native_iconv instead.
 ****************************************************************************/

module tx.cs.native_mbwc;
import tx.cs.interfaces;

private
{
	import std.c.locale;
	import std.c.stddef;
	import std.c.stdlib;
	import std.utf;
	import std.string;
	import std.contracts;

	static this()
	{
		setlocale(LC_CTYPE, "");
	}

	wstring mbstowcs( string s )
	{
		auto r = new wchar_t[s.length+1];
		auto n = std.c.stdlib.mbstowcs(r.ptr, cast(char*) (s~'\0').ptr, r.length);
		r = r[0..n];
		return r.assumeUnique();
	}

	string wcstombs( wstring s )
	{
		auto r = new char[s.length*3+1];
		auto n = std.c.stdlib.wcstombs(r.ptr, cast(wchar_t*) (s~'\0').ptr, r.length);
		r = r[0..n];
		return r.assumeUnique();
	}
}

class NativeCharset : ICharset
{
	static private NativeCharset theInstance;
	static NativeCharset opCall() { return theInstance; }
	static this() { theInstance = new NativeCharset; }

//override:
	string toUTF8( string s )
	{
		return .toUTF8(s.mbstowcs());
	}

	wstring toUTF16( string s )
	{
		static if( is(wchar_t == wchar) ) return s.mbstowcs();
		static if( is(wchar_t == dchar) ) return .toUTF16(s.mbstowcs());
	}

	dstring toUTF32( string s )
	{
		static if( is(wchar_t == wchar) ) return .toUTF32(s.mbstowcs());
		static if( is(wchar_t == dchar) ) return s.mbstowcs();
	}

	string fromUTF8( string s )
	{
		static if( is(wchar_t == wchar) ) return .toUTF16(s).wcstombs();
		static if( is(wchar_t == dchar) ) return .toUTF32(s).wcstombs();
	}

	string fromUTF16( wstring s )
	{
		static if( is(wchar_t == wchar) ) return s.wcstombs();
		static if( is(wchar_t == dchar) ) return .toUTF32(s).wcstombs();
	}

	string fromUTF32( dstring s )
	{
		static if( is(wchar_t == wchar) ) return .toUTF16(s).wcstombs();
		static if( is(wchar_t == dchar) ) return s.wcstombs();
	}
}
